import unittest
import pytest

import datashape
from datashape import dshape, DataShapeSyntaxError


class TestDataShapeStr(unittest.TestCase):
    def test_primitive_measure_str(self):
        self.assertEqual(str(datashape.int8), 'int8')
        self.assertEqual(str(datashape.int16), 'int16')
        self.assertEqual(str(datashape.int32), 'int32')
        self.assertEqual(str(datashape.int64), 'int64')
        self.assertEqual(str(datashape.uint8), 'uint8')
        self.assertEqual(str(datashape.uint16), 'uint16')
        self.assertEqual(str(datashape.uint32), 'uint32')
        self.assertEqual(str(datashape.uint64), 'uint64')
        self.assertEqual(str(datashape.float32), 'float32')
        self.assertEqual(str(datashape.float64), 'float64')
        self.assertEqual(str(datashape.string), 'string')
        self.assertEqual(str(datashape.String(3)), 'string[3]')
        self.assertEqual(str(datashape.String('A')), "string['A']")

    def test_structure_str(self):
        self.assertEqual(str(dshape('{x:int32, y:int64}')),
                         '{x: int32, y: int64}')

    def test_array_str(self):
        self.assertEqual(str(dshape('3*5*int16')),
                         '3 * 5 * int16')

    def test_primitive_measure_repr(self):
        self.assertEqual(repr(datashape.int8),      'ctype("int8")')
        self.assertEqual(repr(datashape.int16),     'ctype("int16")')
        self.assertEqual(repr(datashape.int32),     'ctype("int32")')
        self.assertEqual(repr(datashape.int64),     'ctype("int64")')
        self.assertEqual(repr(datashape.uint8),     'ctype("uint8")')
        self.assertEqual(repr(datashape.uint16),    'ctype("uint16")')
        self.assertEqual(repr(datashape.uint32),    'ctype("uint32")')
        self.assertEqual(repr(datashape.uint64),    'ctype("uint64")')
        self.assertEqual(repr(datashape.float32),   'ctype("float32")')
        self.assertEqual(repr(datashape.float64),   'ctype("float64")')
        self.assertEqual(repr(datashape.string),    'ctype("string")')
        self.assertEqual(repr(datashape.String(3)), 'ctype("string[3]")')
        self.assertEqual(repr(datashape.String('A')),
                         """ctype("string['A']")""")

    def test_structure_repr(self):
        self.assertEqual(repr(dshape('{x:int32, y:int64}')),
                         'dshape("{x: int32, y: int64}")')

    def test_array_repr(self):
        self.assertEqual(repr(dshape('3*5*int16')),
                         'dshape("3 * 5 * int16")')


@pytest.mark.parametrize('s',
                         ['{"./abc": int64}',
                          '{"./a b c": float64}',
                          '{"./a b\tc": string}',
                          '{"./a/[0 1 2]/b/\\n": float32}',
                          pytest.mark.xfail('{"/a/b/0/c\v/d": int8}',
                                            raises=DataShapeSyntaxError),
                          pytest.mark.xfail('{"/a/b/0/c\n/d": int8}',
                                            raises=DataShapeSyntaxError),
                          pytest.mark.xfail('{"/a/b/0/c\r/d": int8}',
                                            raises=DataShapeSyntaxError)])
def test_arbitrary_string(s):
    ds = dshape(s)
    assert dshape(str(ds)) == ds



from datashape.user import *
from datashape import dshape
from datetime import date, time, datetime
import numpy as np

def test_validate():
    assert validate(int, 1)
    assert validate('int', 1)
    assert validate(str, 'Alice')
    assert validate(dshape('string'), 'Alice')
    assert validate(dshape('int'), 1)
    assert validate(dshape('int')[0], 1)
    assert validate('real', 2.0)
    assert validate('2 * int', (1, 2))
    assert not validate('3 * int', (1, 2))
    assert not validate('2 * int', 2)

def test_nested_iteratables():
    assert validate('2 * 3 * int', [(1, 2, 3), (4, 5, 6)])

def test_numeric_tower():
    assert validate(np.integer, np.int32(1))
    assert validate(np.number, np.int32(1))

def test_validate_dicts():
    assert validate('{x: int, y: int}', {'x': 1, 'y': 2})
    assert not validate('{x: int, y: int}', {'x': 1, 'y': 2.0})
    assert not validate('{x: int, y: int}', {'x': 1, 'z': 2})

    assert validate('var * {x: int, y: int}', [{'x': 1, 'y': 2}])

    assert validate('var * {x: int, y: int}', [{'x': 1, 'y': 2},
                                               {'x': 3, 'y': 4}])

def test_tuples_can_be_records_too():
    assert validate('{x: int, y: real}', (1, 2.0))
    assert not validate('{x: int, y: real}', (1.0, 2))

def test_datetimes():
    assert validate('time', time(12, 0, 0))
    assert validate('date', date(1999, 1, 20))
    assert validate('datetime', datetime(1999, 1, 20, 12, 0, 0))

def test_numpy():
    assert validate('2 * int32', np.array([1, 2], dtype='int32'))


def test_issubschema():
    assert issubschema('int', 'int')
    assert not issubschema('int', 'float32')

    assert issubschema('2 * int', '2 * int')
    assert not issubschema('2 * int', '3 * int')

    # assert issubschema('float32', 'real')

def test_integration():
    assert validate('{name: string, arrived: date}',
                    {'name': 'Alice', 'arrived': date(2012, 1, 5)})



import sys
import itertools

# Portions of this taken from the six library, licensed as follows.
#
# Copyright (c) 2010-2013 Benjamin Peterson
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import platform

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

CPYTHON = platform.python_implementation() == 'CPython'

if PY2:
    import __builtin__
    reduce = __builtin__.reduce
    _inttypes = (int, long)
    unicode = __builtin__.unicode
    basestring = __builtin__.basestring
    _strtypes = (str, unicode)

    from types import DictProxyType as MappingProxyType

    if CPYTHON:
        from ctypes import pythonapi, py_object

        mappingproxy = pythonapi.PyDictProxy_New
        mappingproxy.argtypes = [py_object]
        mappingproxy.restype = py_object
        del pythonapi
        del py_object
    else:
        # TODO: Figure out how to make these on pypy.
        # If this gets done, please update the skipif condition in:
        # test_discovery:test_mappingproxy
        def mappingproxy(ob):
            raise ValueError('cannot create mapping proxies in py2 on pypy')

else:
    from functools import reduce
    _inttypes = (int,)
    unicode = str
    basestring = str
    _strtypes = (str,)

    from types import MappingProxyType
    mappingproxy = MappingProxyType


def with_metaclass(metaclass, *bases):
    """Helper for using metaclasses in a py2/3 compatible way.

    Parameters
    ----------
    metaclass : type
        The metaclass to apply.
    bases : iterable of type
        The types to subclass.

    Notes
    -----
    The translations for python 2 and 3 look like:

    ::
        # Compat
        class C(with_metaclass(M, A, B)):
            pass

        # Pyton 2
        class C(A, B):
            __metaclass__ = M

        # Python 3
        class C(A, B, metaclass=M):
            pass
    """
    return metaclass('_', bases, {})


try:
    from collections import OrderedDict
except ImportError:
    class OrderedDict(object):
        def __new__(cls, *args, **kwargs):
            raise TypeError('OrderedDict not supported before python 2.7')
